# include <xc.h>
# include "config.h"

// https://www.cnblogs.com/geekygeek/p/pic32mz_uartcommunication.html
// https://github.com/tiagogala/Projects/blob/master/ua/aula11/lib/uart.c
// ftp://hydro.lin.irk.ru/public/EXP2012/BaikalskSolar/

void Delay10uS(int num)
{
    T2CON = 0x8000; //enable TMR1 with 1:1   prescaler, clock used is FCY = 4000000

    while (num > 0) //wait for specified number of multiples of 100 microseconds
    {
        TMR2 = 0;

        while( TMR2 < (10000000/10000)); //wait 10 microseconds

        num = num-1;
    }
}

void WriteUART1(const char * string) 
{
    while (*string != '\0') 
    {
        while (U1STAbits.UTXBF) {
          ; // wait until tx buffer isn't full
        }
    
        U1TXREG = *string;
    
        ++string;
    }
}

int main(void)
{
    TRISD   &= 0xFFFFF7FF;      //  Enable as output ISO3088 state
    LATD    &= 0xFFFFF7FF;      //  Set ISO3088 to receive state.
    LATD    |= 0x800;           //  Set ISO3088 to send state.

    TRISC   |= 0x4000;          /* RC14 Input */
    TRISC   &= 0xFFFFDFFF;      /* RC13 Output */
    LATC    |= 0x6000;          /* Both RC13 and RC14 HIGH */

    RPC13Rbits.RPC13R = 1;      /* U1TX on RPC13 */
    U1RXRbits.U1RXR   = 7;      /* U1RX on RPC14 */

//  U1BRG  = Baud2BRG( 9600 );  // U1BRG = (FPb / (16*baud)) - 1
    U1BRG  = 650;               // 100000000/(16*9600)-1 = 650
    
    U1MODE   = 0x8000;          // Enable UART for 8-bit data
    U1STASET = 0x1400;          // Enable Transmit and Receive
        
    IFS3bits.U1RXIF = 0;    
    IFS3bits.U1TXIF = 0;

    WriteUART1("Enter text: "); 
    
    while(U1STAbits.TRMT == 0); 
    
    while( 1 )
    {
        char ReceivedChar = 0x00;
        
        LATD &= 0xFFFFF7FF;         //  Set UART<->RS485 IC to receive state.

        while(1)
        {
            /* check for receive errors */
            if(U1STAbits.FERR == 1) { continue; }

            /* must clear the overrun error to keep uart receiving */
            if(U1STAbits.OERR == 1) { U1STAbits.OERR = 0; continue; }

            /* get the data */
            if(U1STAbits.URXDA == 1)
            {
               ReceivedChar = U1RXREG;

               while(U1STAbits.URXDA)
               {
                   char temp = U1RXREG;
               };

               break;
            }
        }

        Delay10uS(100);             // Strange delay 

        LATD |= 0x800;              // Set UART<->RS485 IC to transmit state. 

        while(U1STAbits.UTXBF == 1);

        U1TXREG = ReceivedChar;

        while(U1STAbits.TRMT == 0); 
    }

    return;
}